'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

let _callMethodAsync = (() => {
  var _ref = _asyncToGenerator(function* (url, method, requestBody, requestOptions) {
    const clientId = yield (_Session || _load_Session()).clientIdAsync();
    const user = (yield (_User || _load_User()).default.getCurrentUserAsync()) || {};

    const { idToken, accessToken } = user;
    const skipValidationToken = process.env['EXPO_SKIP_MANIFEST_VALIDATION_TOKEN'];

    let headers = {
      'Exp-ClientId': clientId
    };

    if (skipValidationToken) {
      headers['Exp-Skip-Manifest-Validation-Token'] = skipValidationToken;
    }

    if (idToken) {
      headers['Authorization'] = `Bearer ${idToken}`;
    }

    if (accessToken) {
      headers['Exp-Access-Token'] = accessToken;
    }

    let options = {
      url,
      method: method || 'get',
      headers
    };

    if (requestOptions) {
      options = _extends({}, options, requestOptions);
    }

    if (requestBody) {
      options = _extends({}, options, {
        body: requestBody,
        json: true
      });
    }

    let response = yield (_request || _load_request()).default.promise(options);
    let responseBody = response.body;
    var responseObj;
    if ((_lodash || _load_lodash()).default.isString(responseBody)) {
      try {
        responseObj = JSON.parse(responseBody);
      } catch (e) {
        throw new (_XDLError || _load_XDLError()).default((_ErrorCode || _load_ErrorCode()).default.INVALID_JSON, 'Invalid JSON returned from API: ' + e + '. Response body: ' + responseBody);
      }
    } else {
      responseObj = responseBody;
    }
    if (responseObj.err) {
      let err = ApiError(responseObj.code || 'API_ERROR', 'API Response Error: ' + responseObj.err);
      // $FlowFixMe can't add arbitrary properties to error
      err.serverError = responseObj.err;
      throw err;
    } else {
      return responseObj;
    }
  });

  return function _callMethodAsync(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
})();

let _downloadAsync = (() => {
  var _ref2 = _asyncToGenerator(function* (url, path, progressFunction, retryFunction) {
    let promptShown = false;
    let currentProgress = 0;
    let warningTimer = setTimeout(function () {
      if (retryFunction) {
        retryFunction();
      }
      promptShown = true;
    }, TIMER_DURATION);

    return new Promise(function (resolve, reject) {
      try {
        (0, (_requestProgress || _load_requestProgress()).default)((0, (_request || _load_request()).default)(url, { timeout: TIMEOUT }, function (err) {
          if (err !== null) {
            if (err.code === 'ETIMEDOUT') {
              reject(Error('Server timeout.'));
            } else {
              reject(Error("Couldn't connect to the server, check your internet connection."));
            }
          }
        })).on('progress', function (progress) {
          const roundedProgress = Math.round(progress.percent * 100);
          if (currentProgress !== roundedProgress) {
            currentProgress = roundedProgress;
            clearTimeout(warningTimer);
            if (!promptShown) {
              warningTimer = setTimeout(function () {
                if (retryFunction) {
                  retryFunction();
                }
                promptShown = true;
              }, TIMER_DURATION);
            }
          }
          let percent = progress.percent !== undefined ? progress.percent : 0;
          if (progressFunction) {
            progressFunction(percent);
          }
        }).pipe(_fs.default.createWriteStream(path)).on('finish', function () {
          // Since on('finish') overrides on('progress'), loading bar will never get to 100%
          // this line fixes it.
          if (progressFunction) {
            progressFunction(100);
          }
          clearTimeout(warningTimer);
          resolve();
        }).on('error', reject);
      } catch (e) {
        reject(e);
      }
    });
  });

  return function _downloadAsync(_x5, _x6, _x7, _x8) {
    return _ref2.apply(this, arguments);
  };
})();

require('instapromise');

var _lodash;

function _load_lodash() {
  return _lodash = _interopRequireDefault(require('lodash'));
}

var _request;

function _load_request() {
  return _request = _interopRequireDefault(require('request'));
}

var _requestProgress;

function _load_requestProgress() {
  return _requestProgress = _interopRequireDefault(require('request-progress'));
}

var _fs = _interopRequireDefault(require('fs'));

var _rimraf;

function _load_rimraf() {
  return _rimraf = _interopRequireDefault(require('rimraf'));
}

var _path = _interopRequireDefault(require('path'));

var _FsCache;

function _load_FsCache() {
  return _FsCache = require('./tools/FsCache');
}

var _Config;

function _load_Config() {
  return _Config = _interopRequireDefault(require('./Config'));
}

var _ErrorCode;

function _load_ErrorCode() {
  return _ErrorCode = _interopRequireDefault(require('./ErrorCode'));
}

var _Extract;

function _load_Extract() {
  return _Extract = _interopRequireWildcard(require('./Extract'));
}

var _Session;

function _load_Session() {
  return _Session = _interopRequireWildcard(require('./Session'));
}

var _User;

function _load_User() {
  return _User = _interopRequireDefault(require('./User'));
}

var _UserSettings;

function _load_UserSettings() {
  return _UserSettings = _interopRequireDefault(require('./UserSettings'));
}

var _XDLError;

function _load_XDLError() {
  return _XDLError = _interopRequireDefault(require('./XDLError'));
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const TIMER_DURATION = 30000;
const TIMEOUT = 60000;

function ApiError(code, message) {
  let err = new Error(message);
  // $FlowFixMe error has no property code
  err.code = code;
  // $FlowFixMe error has no property _isApiError
  err._isApiError = true;
  return err;
}

let ROOT_BASE_URL = `${(_Config || _load_Config()).default.api.scheme}://${(_Config || _load_Config()).default.api.host}`;
if ((_Config || _load_Config()).default.api.port) {
  ROOT_BASE_URL += ':' + (_Config || _load_Config()).default.api.port;
}
let API_BASE_URL = ROOT_BASE_URL + '/--/api/';

class ApiClient {

  static callMethodAsync(methodName, args, method, requestBody, requestOptions = {}) {
    return _asyncToGenerator(function* () {
      let url = API_BASE_URL + encodeURIComponent(methodName) + '/' + encodeURIComponent(JSON.stringify(args));
      return _callMethodAsync(url, method, requestBody, requestOptions);
    })();
  }

  static callPathAsync(path, method, requestBody, requestOptions = {}) {
    return _asyncToGenerator(function* () {
      let url = ROOT_BASE_URL + path;
      return _callMethodAsync(url, method, requestBody, requestOptions);
    })();
  }

  static versionsAsync() {
    return _asyncToGenerator(function* () {
      return yield ApiClient._versionCache.getAsync();
    })();
  }

  static xdlSchemaAsync(sdkVersion) {
    return _asyncToGenerator(function* () {
      if (!ApiClient._schemaCaches.hasOwnProperty(sdkVersion)) {
        ApiClient._schemaCaches[sdkVersion] = new (_FsCache || _load_FsCache()).Cacher(_asyncToGenerator(function* () {
          return yield ApiClient.callPathAsync(`/--/xdl-schema/${sdkVersion}`);
        }), `schema-${sdkVersion}.json`, 0, _path.default.join(__dirname, `../caches/schema-${sdkVersion}.json`));
      }

      return yield ApiClient._schemaCaches[sdkVersion].getAsync();
    })();
  }

  static sdkVersionsAsync() {
    return _asyncToGenerator(function* () {
      let versions = yield ApiClient.versionsAsync();
      return versions.sdkVersions;
    })();
  }

  static downloadAsync(url, outputPath, options = {}, progressFunction, retryFunction) {
    return _asyncToGenerator(function* () {
      if (options.extract) {
        let dotExpoHomeDirectory = (_UserSettings || _load_UserSettings()).default.dotExpoHomeDirectory();
        let tmpPath = _path.default.join(dotExpoHomeDirectory, 'tmp-download-file');
        yield _downloadAsync(url, tmpPath);
        yield (_Extract || _load_Extract()).extractAsync(tmpPath, outputPath);
        (_rimraf || _load_rimraf()).default.sync(tmpPath);
      } else {
        yield _downloadAsync(url, outputPath, progressFunction, retryFunction);
      }
    })();
  }
}
exports.default = ApiClient;
ApiClient.host = (_Config || _load_Config()).default.api.host;
ApiClient.port = (_Config || _load_Config()).default.api.port || 80;
ApiClient._versionCache = new (_FsCache || _load_FsCache()).Cacher(_asyncToGenerator(function* () {
  return yield ApiClient.callPathAsync('/--/versions');
}), 'versions.json', 0, _path.default.join(__dirname, '../caches/versions.json'));
ApiClient._schemaCaches = {};
//# sourceMappingURL=__sourcemaps__/Api.js.map
